/* fast_strftim.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/* verschiedene Zeit/Datumsfunktionen.  */

#include "config.h"

#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "portab.h"
#include "mausconlib.h"

/* LIBQUARK_FUNCTION(Funktion,char *,fast_strftime,
[[(char *@var{buf}, size_t @var{buflen}, const char *@var{format}, 
struct tm *@var{tm})]],[[
Die Funktion ist eine schnellere und eingeschrnktere Implementierung
von strftime. Sie beherrscht die folgenden Tokens:

@itemize @bullet
@item %d -- Tag (01..31)
@item %H -- Stunde (00..23)
@item %j -- Tag im Jahr (00..366)
@item %m -- Monat (01..12)
@item %M -- Minuten (00..60)
@item %S -- Sekunden (00..60)
@item %w -- Tag in der Woche, Sonntag == 0 (0..6)
@item %y -- Jahr ohne Jahrhundert (00..99)
@item %Y -- Jahr mit Jahrhundert (0000..9999)
@item %% -- Ein Prozentzeichen.
@end table

Wird als Puffer @var{buf} NULL angegeben, alloziert die Funktion
selbst den bentigten Puffer. 

Diese Funktion hat deutlichen Einflu auf die Ablaufgeschwindigkeit
der Quark.

Liefert bei Erfolg einen Zeiger auf den Puffer, sonst NULL.
]] */

char * 
fast_strftime(char *buf,size_t buflen, const char *format,struct tm *tm)
{
#define HAVEPLACE (scan ? 99 : buflen-(buf-oldbuf))
#define BREAKOUT(n) \
		if (HAVEPLACE<=n) { error_found = TRUE;  goto breakout; }

	char *oldbuf=buf;
	const char *oldformat=format;
	long nr=0;
	long place=0;
	BOOLEAN scan=(buf==NULL);
	BOOLEAN error_found=FALSE;
	size_t need=0;
	BOOLEAN is_nr=FALSE;

retry:
	if (!oldbuf && !scan)
	{
		place=need+1;
		oldbuf=malloc(place);
		if (!oldbuf)
			return NULL;
		format=oldformat;
	}


	while (*format)
	{
		if (*format!='%')
		{
			BREAKOUT(1);
			if (!scan) 
				*buf++=*format++;
			else
			{
				need++;
				format++;
			}
			continue;
		}
		format++;
		switch(*format)
		{
		case 'd': /* Tag */
			nr=tm->tm_mday;
			place=2;
			is_nr=TRUE;
			break;
		case 'H': /* Stunden */
			nr=tm->tm_hour;
			place=2;
			is_nr=TRUE;
			break;
		case 'j': /* Tag im Jahr */
			nr=tm->tm_yday;
			place=3;
			is_nr=TRUE;
			break;
		case 'm': /* Monat */
			nr=tm->tm_mon + 1;
			place=2;
			is_nr=TRUE;
			break;
		case 'M': /* Minuten */
			nr=tm->tm_min;
			place=2;
			is_nr=TRUE;
			break;
		case 'S': /* Sekunden */
			nr=tm->tm_sec;
			place=2;
			is_nr=TRUE;
			break;
		case 'w': /* The day of the week as a decimal number, Sunday being 0 */
			nr=tm->tm_wday;
			place=1;
			is_nr=TRUE;
			break;
		case 'y': /* Year without Century */
			nr=tm->tm_year%100;
			place=2;
			is_nr=TRUE;
			break;
		case 'Y': /* Year with Century */
			nr=tm->tm_year+1900;
			place=4;
			is_nr=TRUE;
			break;

		case '%': /* %% -> % */
			BREAKOUT(1);
			if (!scan) 
				*buf++='%';
			else
				need++;
			format++;
			break;
		default: /* %unknown -> %unknown */
			BREAKOUT(2);
			if (scan) 
				need+=2;
			else
			{
				*buf++='%';
				*buf++=*format++;
			}
			break;
		}
		if (is_nr)
		{
			is_nr=FALSE;
			BREAKOUT(place);
			if (scan)
				need+=place;
			else
			{
				int used_by_nr=place;
				do
				{
					buf[--used_by_nr]='0'+(nr%10);
					nr/=10;
				}
				while (used_by_nr > 0);
				buf+=place;
			}
			format++;
		}
	}
	if (scan)
	{
		scan=FALSE;
		goto retry;
	}
	*buf=0;
breakout:
	if (error_found)
		return NULL;
	return oldbuf;
}

