%{
#include <stdlib.h>
#include <string.h>
#include "rfcaddr.h"

static int yylex(void);
static void yyerror(char *s);
static char * newstring2(const char *a, const char *b);
static char * newstring3(const char *a, const char *b, const char *c);


static char	*errstr;
static char	*comstr;
static char	*cp;
static char	 errbuf[256];
static int	 iseol;

rp_addr_t	*adrlist;
rp_addr_t	*errlist;
%}

%union {
   char	 yChar;
   char	*yString;
   rp_domain_t	*yDom;
   rp_addr_t	*yAddr;
}

%token	EOL ATOM LIT_DOMAIN QUOTED_STRING

%type	<yString>	word domain_ref sub_domain local_part phrase
%type	<yDom>		domain route_list route
%type	<yAddr>		addr_spec route_addr mailbox mbox_list group address

%start	addr_list

%%
addr_list:	addr_lel
       | addr_list addr_lel
       ;

addr_lel: address EOL {
	    $1->comment = comstr;
	    $1->error = errstr;
	    comstr = NULL;
	    errstr = NULL;
	    rp_append_address(&adrlist, $1);
	}
	| address ',' {
	    $1->comment = comstr;
	    $1->error = errstr;
	    comstr = NULL;
	    errstr = NULL;
	    rp_append_address(&adrlist, $1);
	}
	| error {
	    register rp_addr_t	*ap;
	    size_t l;
	    char old=0; /* keep gcc quiet */
	    
	    ap=rp_new_address();
	    l=strlen(cp);
	    if (l>32) {
		old=cp[32];
		cp[32]=0;
	    }
	    sprintf(errbuf, "before \"%s\"\n", cp);
	    if (l>32)
		cp[32]=old;
	    errstr = newstring2(errstr, errbuf);
	    ap->error = errstr;
	    errstr = NULL;
	    comstr = NULL;
	    rp_append_address(&errlist, ap);
	}
	;

address: mailbox {
	    $$ = $1;
	}
	| group {
	    $$ = $1;
	}
	;

group	: phrase ':' mbox_list ';' {
	    register rp_addr_t	*a;

	    for (a = $3; a; a = a->next)
		a->group = $1;
	    $$ = $3;
	}
	;

mbox_list: mailbox {
	    $$ = $1;
	}
	| mbox_list ',' mailbox {
	    $3->comment = comstr;
	    $3->error = errstr;
	    comstr = NULL;
	    errstr = NULL;
	    rp_append_address(&($1), $3);
	    $$ = $1;
	}
	;

mailbox: addr_spec {
	    $$ = $1;
	}
	| route_addr {
	    $$ = $1;
	}
	| phrase route_addr {
	    $2->name = $1;
	    $$ = $2;
	}
	;

phrase	: word {
	    $$ = $1;
	}
	| phrase word {
	    $$ = newstring3($1, " ", $2);
	    free($1);
	    free($2);
       }
       ;

route_addr: '<' addr_spec '>' {
	   $$ = $2;
       }
       | '<' route addr_spec '>' {
	   rp_prepend_domain(&($3->route), $2);
	   $$ = $3;
       }
       ;

route	: route_list ':' {
	    $$ = $1;
	}
	;

route_list: '@' domain {
	    $$ = $2;
	}
	| route_list ',' '@' domain {
	    rp_append_domain(&($1), $4);
	    $$ = $1;
	}
	;

addr_spec: local_part '@' domain {
	    register rp_addr_t	*ap;

	    $$ = ap = rp_new_address();
	    ap->localp = $1;
	    ap->destdom = $3;
	    ap->route = $3;
	}
	| local_part {
	    register rp_addr_t	*ap;

	    $$ = ap = rp_new_address();
	    ap->localp = $1;
	    ap->destdom = NULL;
	}
	;


local_part: word {
		$$ = $1;
	    }
	    | local_part '.' word {
		$$ = newstring3($1, ".", $3);
		free($1);
		free($3);
	    }
	    | local_part '%' word {
		$$ = newstring3($1, "%", $3);
		free($1);
		free($3);
	    }
	    ;

domain	: sub_domain {
	    register rp_domain_t	*dp;

	    dp = rp_new_domain();
	    dp->sub[0] = $1;
	    dp->top = dp->sub;
	    $$ = dp;
	}
	| domain '.' sub_domain {
	    ($1->top)++;
	    *($1->top) = $3;
	    $$ = $1;
	}
	;

sub_domain: domain_ref {
	    $$ = $1;
	}
	| LIT_DOMAIN {
	    $$ = yylval.yString;
	}
	;

domain_ref: ATOM {
	    $$ = yyval.yString;
	}
	;

word	: ATOM {
	    $$ = yylval.yString;
	}
	| QUOTED_STRING {
	    $$ = yylval.yString;
	}
	;

%%

#include <stdio.h>
#include <ctype.h>

#define ERROR	-2


static char *
newstring3(const char *a, const char *b, const char *c)
{
    char	*p;
    char	*q;
    size_t	need;
    size_t	a_len;
    size_t	b_len;
    size_t	c_len;

    if (!a)
    {
        a="";
        a_len=0;
    }
    else
    	a_len=strlen(a);

    if (!b)
    {
        b="";
    	b_len=0;
    }
    else
    	b_len=strlen(b);

    if (!c)
    {
        c="";
        c_len=0;
    }
    else
    	c_len=strlen(c);

    need = a_len + b_len + c_len + 1;
    if ((p = malloc((MALLOCT)need)) == NULL) {
		perror("newstring3 (malloc)");
		exit(1);
    }
    strcpy(p, a);
    q = p + a_len;
    strcpy(q, b);
    q += b_len;
    strcpy(q, c);
    /* Uwe Ohse, 21.3.1994: was return(q) */
    return(p);
}


static char *
newstring2(const char *a, const char *b)
{
    char		*p;
    size_t	need;
    size_t  a_len;
    size_t  b_len;

    if (!a)
    {
        a="";
        a_len=0;
    }
    else
    	a_len=strlen(a);

    if (!b)
    {
        b="";
        b_len=0;
    }
 	else
 		b_len=strlen(b);

    need = a_len + b_len + 1;
    if ((p = malloc((MALLOCT)need)) == NULL) {
		perror("newstring2 (malloc)");
		exit(1);
    }
    strcpy(p, a);
    strcpy(p+a_len, b);
    return(p);
}


static void
yyerror(char *s)
{
   size_t l;
   char old=0; /* keep gcc happy */
   switch(yychar) {
       default:
	   sprintf(errbuf, "%s: \"%c\" unexpected\n", s, yylval.yChar);
	   errstr = newstring2(errstr, errbuf);
	   break;
       case LIT_DOMAIN:
       case QUOTED_STRING:
       case ATOM:
	   l=strlen(yylval.yString);
	   if (l>32) {
	       old=yylval.yString[32];
	       yylval.yString[32]=0;
	   }
	   sprintf(errbuf, "%s: \"%s\" unexpected\n", s, yylval.yString);
	   if (l>32)
	       yylval.yString[32]=old;
	   errstr = newstring2(errstr, errbuf);
	   break;
       case 0: /* EOF */
	   sprintf(errbuf, "%s: unexpected EOF\n", s);
	   errstr = newstring2(errstr, errbuf);
	   break;
   }

}


void rfc_parse_address(char *line)
{
    adrlist = NULL;
    errlist = NULL;
	/* well, it shouldn't happen, but it *does*. I don't like this code,
	 * but for the moment i have no serious choice.
	 */
	if (*line=='@' && strchr(line,':'))
	{
		char *kopie=malloc(strlen(line+3));
		*kopie='<';
		strcpy(kopie+1,line);
		strcat(kopie,">");
		cp=kopie;
    	(void)yyparse();
    	free(kopie);
	}
	else
	{
    	cp = line;
    	(void)yyparse();
    }
}

static int
yylex(void)
{
    register char	*p;
    register int	 paren_count;

    while (isascii(*cp) && (isspace(*cp) || (*cp == '('))) {
	if (*cp == '(') {
	    char *combuf;
	    for (paren_count = 1, p = cp + 1; paren_count; p++) {
		if (*p == '\0')
		    return(EOF);
		if (*p == '(')
		    paren_count++;
		else if( *p == ')')
		    paren_count--;
	    }
	    combuf=malloc ( (p-2) - cp + 1);
	    if (combuf == NULL) {
		perror("malloc failure");
		exit(1);
	    }
	    memcpy(combuf, cp + 1, (p - 2) - cp);
	    combuf[(p-2) - cp]='\0';
	    if (comstr == NULL) 
		comstr = combuf;
	    else {
		comstr = newstring3(comstr, ", ", combuf);
		free(combuf);
	    }
	    cp = p;
	}
	else
	    cp++;
    }

    if (!isascii(*cp))
	return(ERROR);

    switch (*cp) {
	case '\0':
	    if (iseol) {
		iseol = 0;
		return(EOF);
	    }
	    iseol = 1;
	    return(EOL);
	case '.':
	case ',':
	case ':':
	case ';':
	case '@':
	case '%':
	case '<':
	case '>':
	    yylval.yChar = *cp;
	    return(*cp++);
	case '[':       /* LIT_DOMAIN */
	    for (p = cp; *++p && *p != ']'; )
		;
	    if (*p == '\0')
		return(EOF);
	    if ((yylval.yString = malloc((MALLOCT)(p - cp + 2))) == NULL) {
		fprintf(stderr,"not enough memory\n");
		return(ERROR);
	    }
	    strncpy(yylval.yString, cp, p - cp + 1);
	    yylval.yString[p - cp + 2] = '\0';
	    cp = ++p;
	    return(LIT_DOMAIN);
	case '"':       /* QUOTED_STRING */
	    for (p = cp; *++p && *p != '"'; )
		;
	    if (*p == '\0')
		return(EOF);
	    if ((yylval.yString = malloc((MALLOCT)(p - cp + 2))) == NULL) {
		fprintf(stderr,"not enough memory\n");
		return(ERROR);
	    }
	    strncpy(yylval.yString, cp, p - cp + 1);
	    /* uo, 24.3.1994, was "+ 2", out of malloced area */
	    yylval.yString[p - cp + 1] = '\0';
	    cp = ++p;
	    return(QUOTED_STRING);
    }
    for (p = cp; ; p++)
	switch (*p) {
	    case 'a':
	    case 'A':
		if ((p[1] == 't' || p[1] == 'T') && isspace(p[2])
			&& isalpha(p[3])) {
		    yylval.yChar = '@';
		    cp = p + 2;
		    return('@');
		}
		break;
	    case '.':
	    	if (isspace(p[1]))
	    		break;
	    case ',':
	    case ':':
	    case ';' :
	    case '@':
	    case '%':
	    case '<':
	    case '>':
	    case '(':
	    case ')':
	    case '[':
	    case '"':
	    case '\0':
		goto out;
	    default:
		if (isspace(*p))
		    goto out;
	}
out:
	if (p==cp) {
	fprintf(stderr,"loop at symbol %c\n",*cp);
	return(ERROR);
	}
    if ((yylval.yString = malloc((MALLOCT)(p - cp + 1))) == NULL) {
	fprintf(stderr,"not enough memory\n");
	return(ERROR);
    }
    strncpy(yylval.yString, cp, p - cp);
    yylval.yString[p - cp] = '\0';
    cp = p;
    return(ATOM);
}
