/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1993 1994 1995 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
   do_conf.c - Auslesen der Konfigurationsdatei des Pakets.

*/

#include <stdio.h>

#include "mausconsys.h"

#include <conflib.h>
#include <syslog.h>

#include "mausconlib.h"
#include "mauscon.h"
#define INIT(x,y)	y,
#include "mauscon.h"
#undef INIT

const char *action;
const char *program_name;

static cl_enum_t workmodeenum[] =
{
	{"user", USER},
	{"gate", GATE},
	{NULL, 0}
};

static cl_enum_t rekillenum[] =
{
	{"full", REKILL_FULL},
	{"smart", REKILL_SMART},
	{"none", REKILL_NONE},
	{NULL, 0}
};

static cl_enum_t boxtypenum[] =
{
	{"quark", BOXTYP_QUARK},
	{"quark 2", BOXTYP_QUARK},
	{"quark II", BOXTYP_QUARK},
	{"maus", BOXTYP_MAUS},
	{NULL, 0}
};

cl_enum_t facenum[] =
{
	{"daemon", LOG_DAEMON},
	{"user", LOG_USER},
	{"uucp", LOG_UUCP},
	{"mail", LOG_MAIL},
	{"news", LOG_NEWS},
	{"local0", LOG_LOCAL0},
	{"local1", LOG_LOCAL1},
	{"local2", LOG_LOCAL2},
	{"local3", LOG_LOCAL3},
	{"local4", LOG_LOCAL4},
	{"local5", LOG_LOCAL5},
	{"local6", LOG_LOCAL6},
	{"local7", LOG_LOCAL7},
	{NULL, 0}
};

/* Die folgenden sind in mausconlib.h auf jeden Fall definiert worden */
cl_enum_t prioenum[] =
{
	{"debug", LOG_DEBUG},
	{"info", LOG_INFO},
	{"notice", LOG_NOTICE},
	{"warning", LOG_WARNING},
	{"err", LOG_ERR},
	{"crit", LOG_CRIT},
	{"alert", LOG_ALERT},
	{"emerg", LOG_EMERG},
	{NULL, 0}
};


cl_list_t *list_allowed;
static cl_ulist_t *unknowns=NULL;

cl_var_t Tab[] =
{
	{"mode", &workmodeenum, CL_ENUM, &opts.workmode, 0, NULL},
	{"workmode", "mode", CL_ALIAS, NULL, 0, NULL},

	{"spoolbase", NULL, CL_DIRNAME, &opts.spoolbase, 0, NULL},
	{"tempdir", "TMP", CL_DIRNAME, &opts.tempdir, CL_POSTENV, NULL},

	{"sl-facility", &facenum, CL_ENUM, &opts.sl_facility, 0, NULL},
	{"syslog-facility", "sl-facility", CL_ALIAS, NULL, 0, NULL},
	{"sl-priority", &prioenum, CL_ENUM, &opts.sl_priority, 0, NULL},
	{"syslog-priority", "sl-priority", CL_ALIAS, NULL, 0, NULL},
	{"smart-LOG", NULL, CL_BOOLEAN, &opts.smart_LOG, 0, NULL},

	{"gatemaster", NULL, CL_STRING, &opts.gatemaster, 0, NULL},
	{"mysitedomain", NULL, CL_STRING, &opts.mysitedomain, 0, ".*\\..*"},
	{"mailuserdomain", NULL, CL_STRING, &opts.mailuserdomain, 0, NULL},
	{"mailuser", "mailuserdomain", CL_ALIAS, NULL, 0, NULL},
	{"allowed", " \t", CL_LIST, &list_allowed, 0, NULL},

	{"boxtyp", &boxtypenum, CL_ENUM, &opts.boxtyp, 0, NULL},
	{"pollsite", NULL, CL_STRING, &opts.pollsite, 0, NULL},
	{"pollsitedomain", NULL, CL_STRING, &opts.pollsitedomain, 0, NULL},
	{"mausdomain", NULL, CL_STRING, &opts.mausdomain, 0, "\\.maus\\.de"},
	{"tabsize", NULL, CL_NUMSHORT, &opts.tabsize, 0, NULL},
	{"split-after", NULL, CL_NUM, &opts.split_after, 0, NULL},
	{"subjectlen", NULL, CL_NUMSHORT, &opts.subjectlen, 0, NULL},
	{"autoinfosubscribe", NULL, CL_BOOLEAN, &opts.autoinfosubscribe, 0, NULL},
	{"rekillmode", &rekillenum, CL_ENUM, &opts.rekillmode, 0, NULL},
	{"send-crosspostings", NULL, CL_BOOLEAN, &opts.send_crosspostings, 0, NULL},
	{"send-followup-to", NULL, CL_BOOLEAN, &opts.send_followup_to, 0, NULL},
	{"send-sender", NULL, CL_BOOLEAN, &opts.send_sender, 0, NULL},
	{"send-reply-to", NULL, CL_BOOLEAN, &opts.send_reply_to, 0, NULL},
	{"send-path", NULL, CL_BOOLEAN, &opts.send_path, 0, NULL},
	{"generate-empty-mail-body", NULL, CL_BOOLEAN, &opts.generate_empty_mail_body, 0, NULL},
	{"cancel-original", NULL, CL_BOOLEAN, &opts.cancel_original, 0, NULL},
	{"strip-bangpath", NULL, CL_BOOLEAN, &opts.strip_bangpath, 0, NULL},
	{"strip-domain", NULL, CL_BOOLEAN, &opts.strip_domain, 0, NULL},
	{"mail-status-change", NULL, CL_BOOLEAN, &opts.mail_status_change, 0, NULL},
	{"autostatus", NULL, CL_CHAR, &opts.autostatus, 0, "[ZGzgyY]"},
	{"t-online-id-hack", NULL, CL_BOOLEAN, &opts.t_online_id_hack, 0, NULL},

	{"direct-smtp", NULL, CL_BOOLEAN, &opts.direct_smtp, 0, NULL},
	{"mailhost", NULL, CL_STRING, &opts.mailhost, 0, NULL},
	{"reroute-host", NULL, CL_STRING, &opts.reroute_host, 0, NULL},
	{"newshost", NULL, CL_STRING, &opts.newshost, 0, NULL},
	{"path-entry", NULL, CL_STRING, &opts.path_entry, 0, NULL},
	{"gprefix", NULL, CL_STRING, &opts.gprefix, 0, NULL},
	{"group-prefix", "gprefix", CL_ALIAS, NULL, 0, NULL},
	{"group-name-prefix", "gprefix", CL_ALIAS, NULL, 0, NULL},
	{"dist-local", "usenet-dist-lokal", CL_ALIAS, NULL, 0, NULL},
	{"dist-lokal", "usenet-dist-lokal", CL_ALIAS, NULL, 0, NULL},
	{"usenet-dist-local", "usenet-dist-lokal", CL_ALIAS, NULL, 0, NULL},
	{"usenet-dist-lokal", NULL, CL_STRING, &opts.usenet_dist_lokal, 0, NULL},
	{"usenet-dist-mausnet", NULL, CL_STRING, &opts.usenet_dist_mausnet, 0, NULL},
	{"dist-mausnet", "usenet-dist-mausnet", CL_ALIAS, NULL, 0, NULL},
	{"warn-news-routing", NULL, CL_BOOLEAN, &opts.warn_news_routing, 0, NULL},
	{"unlink-unallowed", NULL, CL_BOOLEAN, &opts.unlink_unallowed, 0, NULL},
	{"max-references", NULL, CL_NUMSHORT, &opts.max_references, 0, NULL},
	{"enable-dsn-bounces", NULL, CL_BOOLEAN, &opts.enable_dsn_bounces, 0, NULL},

	{"unconvertable-from-kill", NULL, CL_BOOLEAN, &opts.unconvertable_from_kill, 0, NULL},
	{"unconvertable-from-path", NULL, CL_STRING, &opts.unconvertable_from_path, 0, NULL},
	{"unconvertable-from-distribution", NULL, CL_STRING, &opts.unconvertable_from_distribution, 0, NULL},
	{"foreign-export-kill", NULL, CL_BOOLEAN, &opts.foreign_export_kill, 0, NULL},
	{"foreign-export-path", NULL, CL_STRING, &opts.foreign_export_path, 0, NULL},
	{"foreign-export-distribution", NULL, CL_STRING, &opts.foreign_export_distribution, 0, NULL},

	{"mime-coded-headers", NULL, CL_BOOLEAN, &opts.mime_coded_headers, 0, NULL},
	{"mime-coded-body", NULL, CL_BOOLEAN, &opts.mime_coded_body, 0, NULL},
	{"mime-headerlines", NULL, CL_BOOLEAN, &opts.mime_headerlines, 0, NULL},

	{"rmail-route", NULL, CL_STRING, &opts.rmail_route, 0, NULL},
	{"rsmtp-route", NULL, CL_STRING, &opts.rsmtp_route, 0, NULL},
	{"rnews-route", NULL, CL_STRING, &opts.rnews_route, 0, NULL},

	{"keep-sent", NULL, CL_BOOLEAN, &opts.keep_sent, 0, NULL},
	{"compress", NULL, CL_STRING, &opts.compress, 0, NULL},
	{"lock-tries", NULL, CL_NUMSHORT, &opts.lock_tries, 0, NULL},
	{"lock-sleep", NULL, CL_NUMSHORT, &opts.lock_sleep, 0, NULL},
	{"diffcmd", NULL, CL_STRING, &opts.diffcmd, 0, NULL},

	{NULL, NULL, 0, NULL, 0}
};

static cl_var_t *backup;

const char *
get_conf_value (const char *s)
{
	cl_var_t *t = Tab;
	int found = 0;
	char buf[128];
	cl_ulist_t *unk;
	const char *erg = NULL;

	while (t->varname != NULL)
	{
		if (!strcasecmp (t->varname, s))
		{
			found++;
			switch (t->typ)
			{
			case CL_ENUM:
				{
					const cl_enum_t *p;
					int found_enums = 0;

					p = (const cl_enum_t *) t->secdata;
					while (p && p->magic)
					{
						if (*(long *) t->adr == p->value)
						{
							found_enums = 1;
							erg = p->magic;
							break;
						}
						p++;
					}
					if (!found_enums)
						erg = ((const cl_enum_t *) t->secdata)->magic;
				}
				break;
			case CL_STRING:
			case CL_DIRNAME:
				erg = *(char **) t->adr;
				break;
			case CL_NUM:
				sprintf (buf, "%ld", *(long *) t->adr);
				erg = buf;
				break;
			case CL_NUMSHORT:
				sprintf (buf, "%d", *(short *) t->adr);
				erg = buf;
				break;
			case CL_BOOLEAN:
				if (*(int *) t->adr)
					erg = "true";
				else
					erg = "false";
				break;
			case CL_ARRAY:
				erg = "?";
				break;
			case CL_ALIAS: 
				return get_conf_value((const char *)t->secdata);
			case CL_LIST:
				erg= "?";
				break;
			case CL_DUMMY:
				erg = "?";
				break;
			case CL_CHAR:
				sprintf (buf, "%c", *(char *) t->adr);
				erg = buf;
				break;
			}
		}						/* if */
		t++;
	}							/* while */
	if (found)
		return erg;
	for (unk=unknowns; unk; unk=unk->next) {
		if (!strcasecmp (unk->varname, s)) 
			return unk->inhalt;
	}

	return "(not defined)";
}

void
dump_conf (FILE * f)
{
	fprintf (f, "action: %s\n", action);
	cl_writeconf (f, Tab);
}

static void 
list_convert(mc_list_t **mclist,cl_list_t *cllist)
{
	mc_list_t *new;
	mc_list_t *last=NULL;
	cl_list_t *akt;
	akt=cllist;
	while (akt)
	{
		if (akt->inhalt)
		{
			new=XMALLOC(sizeof(mc_list_t));
			new->data=XSTRDUP(akt->inhalt);
			new->ext=NULL;
			new->next=NULL;
			if (last)
				last->next=new;
			else
				*mclist=new;
			last=new;
		}
		akt=akt->next;
	}
}

static void 
list_free(mc_list_t **mclist)
{
	mc_list_t *akt;
	mc_list_t *next;
	akt=*mclist;
	while (akt)
	{
		if (akt->data)
			free(akt->data);
		if (akt->ext)
			free(akt->ext);
		next=akt->next;
		free(akt);
		akt=next;
	}
	*mclist=NULL;
}

int
do_conf (const char *fname, const char *aktion)
{
	int erg;
	int is_routing;
	cl_file_t *cf;
	char *t;
	char *tmp;

	static int first=0;

	tmp = XSTRDUP(aktion);
	for (t=tmp;*t;t++)
	{
		if (islower(*t))
			*t=tolower(*t);
	}
	action=tmp;

	is_routing= !strcasecmp(aktion,"route");

	/* need at least 0.4.0 */
	if (cl_getversion () < 400)
	{
		log (LOG_CRIT, _("incompatible conflib version: %d"), 
			 cl_getversion ());
		return 1;
	}
	if (!first)
	{
		backup=XMALLOC(sizeof(Tab));
		memcpy(backup,Tab,sizeof(Tab));
		first=1;
	}
	else
	{
		cl_clearconf(Tab);
		memcpy(Tab,backup,sizeof(Tab));
	}

	cf=cl_openconf(fname,"r");
	if (!cf)
	{
		/* tja, pech */
		log (LOG_ERR, _("cannot open %s: %s"), fname,strerror(errno));
		return 1;
	}

	erg=cl_getstanza(cf, Tab, "default",0,&unknowns);
	if (erg)
	{
		/* tja, pech */
		cl_closeconf(cf);
		log (LOG_ERR, _("cannot read `%s' from %s"), "default", fname);
		return 1;
	}
	if (!is_routing && aktion)
	{
		erg=cl_getstanza(cf, Tab, aktion,0,&unknowns);
		if (erg)
		{
			/* tja, pech */
			cl_closeconf(cf);
			log (LOG_ERR, _("cannot read `%s' from %s"), aktion, fname);
			return 1;
		}
	}
	cl_closeconf(cf);
	if (!opts.mysitedomain)
	{
#ifdef HAVE_GETHOSTNAME
		char tmp_p[256];
		if (gethostname (tmp_p, sizeof(tmp_p)-1)!=0)
		{
			fprintf(stderr,_("gethostname failed: %s\n"),strerror(errno));
			return 1;
		}
		opts.mysitedomain = XSTRDUP(tmp_p);
#else
		fprintf(stderr,("%s: missing opts.mysitedomain\n"), fname);
		return 1;
#endif
	};
#define MISSCHECK(x) \
if (!x || !*x) \
{ \
	fprintf(stderr,_("%s: missing %s\n"),fname,#x); \
	log(LOG_ERR,"%s missing",#x); \
	return 1; \
}
	MISSCHECK (opts.mausdomain);
	MISSCHECK (opts.mysitedomain);
	MISSCHECK (opts.spoolbase);
	if (!is_routing)
	{
		MISSCHECK (opts.mailuserdomain);
		if (!strchr(opts.mailuserdomain,'@'))
		{
			char *tmp_p;
			tmp_p =concat(opts.mailuserdomain,"@",opts.mysitedomain,NULL);
			XNULLCHECK(tmp_p);
			opts.mailuserdomain=tmp_p;
		}
		MISSCHECK (opts.pollsite);
		MISSCHECK (opts.gprefix);
#if 0
		MISSCHECK (opts.path_entry);
#endif
#ifdef WANT_SMTP
		if (!opts.mailhost)
			opts.mailhost=XSTRDUP(opts.mysitedomain);
#endif
#ifdef WANT_NNTP
		if (!opts.newshost)
			opts.newshost=XSTRDUP(opts.mysitedomain);
#endif
	/* Zwischen conflib-Listenformat und MC-Listenformat wandeln */
		list_free(&opts.allowed);
		list_convert(&opts.allowed,list_allowed);
	}
	if (!opts.pollsitedomain || !*opts.pollsitedomain)
	{
		char buf[1024];
		sprintf(buf,"%s%s",opts.pollsite,opts.mausdomain);
		opts.pollsitedomain=XSTRDUP(buf);
	}
	MISSCHECK (opts.gatemaster);
	if (!strchr(opts.gatemaster,'@'))
	{
		char *tmp_p;
		tmp_p =concat(opts.gatemaster,"@",opts.mysitedomain,NULL);
		XNULLCHECK(tmp_p);
		opts.gatemaster=tmp_p;
	}
#undef MISSCHECK
	return 0;
}
