/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1994, 1995 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   domains.c: Domainverwaltung fr MausConnect
*/

#include "mausconsys.h"

#include <stdio.h>
#include <syslog.h>

#include "mausconlib.h"
#include "mauscon.h"
#include "domains.h"

static box_t *b_anker;
static fidodomain_t *f_anker;

static BOOLEAN
dom_read_itb(FILE *f, char *buf, size_t bufsize)
{
	box_t *b=NULL;
	while (fgets(buf,bufsize,f))
	{
		char *p;
		p=strpbrk(buf,"\r\n");
		if (p)
			*p=0;
		if (!b && *buf!='*')
			continue;
		switch(*buf)
		{
		case '*': 
			b=XMALLOC(sizeof(box_t));
			b->next=b_anker;
			b_anker=b;
			b->kuerzel=XSTRDUP(buf+1);
			b->vglkuerzel=entumlauten1(buf+1);
			b->secdomain=NULL;
			b->primdomain=NULL;
			b->orga=NULL;
			XNULLCHECK(b->vglkuerzel);
			break;
		case 'D':
			if (buf[1]=='.') {
				char buf2[1024];
				sprintf(buf2,"%s%s",b->kuerzel,buf+1);
				b->primdomain=entumlauten1(buf2);
			} else {
				b->primdomain=entumlauten1(buf+1);
			}
			XNULLCHECK(b->primdomain);
			break;
		case 'Z':
			{
				mc_list_t *ele=XMALLOC(sizeof(mc_list_t));
				ele->ext=NULL;
				ele->next=b->secdomain;
				b->secdomain=ele;
				if (buf[1]=='.') {
					char buf2[1024];
					sprintf(buf2,"%s%s",b->kuerzel,buf+1);
					ele->data=entumlauten1(buf2);
				} else {
					ele->data=entumlauten1(buf+1);
				}
				XNULLCHECK(ele->data);
			}
			break;
		case 'N':
			b->orga=XSTRDUP(buf+1);
			break;
		case 'T':
			if (b->orga) {
				char *t=concat(b->orga, " ", buf+1,NULL);
				XNULLCHECK(t);
				free(b->orga);
				b->orga=t;
			}
		}
	}
	return TRUE;
}

BOOLEAN 
dom_init(void)
{
	char fname[PATH_MAX];
	char buf[4096];
	char *p,*q;
	FILE *f;

	sprintf(fname,"%s/%s/infofiles/ITS",opts.spoolbase,action);
	f=fopen(fname,"r");
	if (!f) {
		sprintf(fname,"%s/%s/infofiles/ITB",opts.spoolbase,action);
		f=fopen(fname,"r");
	}
	if (f) {
		if (!dom_read_itb(f,buf,sizeof(buf)))
		{
			fclose(f);
			return FALSE;
		}
		fclose(f);
	} else {

		sprintf(fname,"%s/%s/etc/domains",opts.spoolbase,action);
		f=fopen(fname,"r");
		if (!f) {
			int s=errno;
			sprintf(fname,"%s/domains",opts.spoolbase);
			f=fopen(fname,"r");
			if (!f) {
				sprintf(fname,"%s/%s/etc/domains",opts.spoolbase,action);
				log (LOG_NOTICE,_("cannot read %s: %s"),fname,strerror(s));
				return FALSE;
			}
		}

		while (fgets(buf,sizeof(buf),f))
		{
			box_t *b;
			mc_list_t *sec;
			p=strpbrk(buf,"\r\n");
			if (p)
				*p='\0';
			if (isspace(*buf))
			{
				if (!b_anker)
					continue;
				p=buf;
				while(isspace(*p))
					p++;
				if (*p=='O')
					b_anker->orga=strdup(p+1);
				continue;
			}
			p=strpbrk(buf,"\t ");
			if (p)
				*p++='\0';
			b=XMALLOC(sizeof(box_t));
			b->next=b_anker;
			b_anker=b;
			b->kuerzel=XSTRDUP(buf);
			b->vglkuerzel=entumlauten1(buf);
			XNULLCHECK(b->vglkuerzel);
			b->secdomain=NULL;
			b->primdomain=NULL;
			b->orga=NULL;
			if (!p)
				continue;
			while (isspace(*p))
				p++;
			q=strpbrk(p,"\t ");
			if (q)
				*q++='\0';
			b->primdomain=XSTRDUP(p);

			p=q;
			while (p)
			{
				while (isspace(*p))
					p++;
				q=strpbrk(p,"\t ");
				if (q)
					*q++='\0';
				sec=XMALLOC(sizeof(mc_list_t));
				sec->ext=NULL;
				sec->next=b->secdomain;
				b->secdomain=sec;
				sec->data=XSTRDUP(p);
				p=q;
			}
		}
		fclose(f);
	}

	sprintf(fname,"%s/fidodomains",opts.spoolbase);
	f=fopen(fname,"r");
	if (!f)
		return TRUE; /* fidodomains ist optional */

	/* Format:
	 * 1 fidonet.org Fido FidoNet
	 * ..
	 * 70 - VPLNET         (keine Internetdomain bekannt)
	 */
	while (fgets(buf,sizeof(buf),f))
	{
		char *zonep;
		char *idomainp;
		char *fdomainp1;
		char *fdomainp2;
		fidodomain_t *fido;
		p=strpbrk(buf,"\r\n");
		if (p)
			*p='\0';
		zonep=buf;
		while(isspace(*zonep))
			zonep++;
		if (*zonep=='#' || !isdigit(*zonep))
			continue;
		idomainp=strpbrk(zonep,"\t ");
		if (!idomainp)
			continue;
		*idomainp++=0;
		while(isspace(*idomainp))
			idomainp++;
		strlwr(idomainp);

		fdomainp1=strpbrk(idomainp,"\t ");
		if (!fdomainp1)
			continue;
		*fdomainp1++=0;
		while(isspace(*fdomainp1))
			fdomainp1++;
		strlwr(fdomainp1);

		fdomainp2=strpbrk(fdomainp1,"\t ");
		if (fdomainp2)
		{
			*fdomainp2++=0;
			while(isspace(*fdomainp2))
				fdomainp2++;
			strlwr(fdomainp2);
		}

		fido=XMALLOC(sizeof(fidodomain_t));
		fido->next=f_anker;
		f_anker=fido;
		fido->zone=(int) strtol(zonep,NULL,10);
		fido->internet_domain=XSTRDUP(idomainp);
		fido->fido_domain1=XSTRDUP(fdomainp1);
		if (fdomainp2)
			fido->fido_domain2=XSTRDUP(fdomainp2);
		else
			fido->fido_domain2=NULL;
	}
	fclose(f);
	return TRUE;
}

box_t *
dom_find_boxkuerzel(const char *kuerzel)
{
	char buf[128];
	box_t *akt;
	entumlauten(kuerzel,buf);
	for (akt=b_anker;akt;akt=akt->next)
	{
		if (!strcasecmp(buf,akt->vglkuerzel))
			return akt;
	}
	return NULL;
}

box_t *
dom_find_domain(const char *domain)
{
	box_t *akt;
	mc_list_t *sec;
	char *canon=entumlauten1(domain);
	XNULLCHECK(canon);
	
	for (akt=b_anker;akt;akt=akt->next)
	{
		if (akt->primdomain && !strcasecmp(canon,akt->primdomain))
		{
			free(canon);
			return akt;
		}
		for (sec=akt->secdomain;sec;sec=sec->next)
		{
			if (!strcasecmp(canon,sec->data))
			{
				free(canon);
				return akt;
			}
		}
	}
	free(canon);
	return NULL;
}

fidodomain_t *
fido_find_zone(int zone)
{
	fidodomain_t *akt;
	
	for (akt=f_anker;akt;akt=akt->next)
	{
		if (akt->zone==zone)
			return akt;
	}
	return NULL;
}

fidodomain_t *
fido_find_idomain(const char *idomain)
{
	fidodomain_t *akt;
	char *copy=XSTRDUP(idomain);
	strlwr(copy);
	
	for (akt=f_anker;akt;akt=akt->next)
	{
		const char *p;
		p=strstr(copy,akt->internet_domain);
		if (p)
		{
			if (p[strlen(akt->internet_domain)]=='\0')
				return akt;
		}
	}
	free(copy);
	return NULL;
}

fidodomain_t *
fido_find_fdomain(const char *fdomain)
{
	fidodomain_t *akt;
	char *copy=XSTRDUP(fdomain);
	strlwr(copy);
	
	for (akt=f_anker;akt;akt=akt->next)
	{
		if (*copy==*akt->fido_domain1
			&& 0==strcmp(copy,akt->fido_domain1))
		{
			free(copy);
			return akt;
		}
		if (akt->fido_domain2 && *copy==*akt->fido_domain2
			&& 0==strcmp(copy,akt->fido_domain2))
		{
			free(copy);
			return akt;
		}
	}
	free(copy);
	return NULL;
}
