
/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1992 1993 1994 1995 Michael Kraehe, Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   info.c: Dealing with infofiles.
*/

#include "mausconsys.h"

#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <ctype.h>
#include <limits.h>
#include <syslog.h>
#include <unistd.h>

#include "mausconlib.h"
#include "mauscon.h"
#include "maus.h"

static minfo_t *anker;
static BOOLEAN dirty;

int info_read_list(void)
{
	char fname[PATH_MAX];
	char str[1024];
	minfo_t *new;
	FILE *f;
	sprintf(fname,"%s/%s/etc/info.list",opts.spoolbase,action);
	f=fopen(fname,"r");
	if (!f)
	{
		log(LOG_WARNING,_("cannot load %s"),fname);
		return 0; /* naja, nur ein kleinerer Fehler! */
	}
	new=NULL;
	while (fgets (str, sizeof(str), f))
	{
		char *p=strpbrk(str,"\r\n");
		char *inhalt;
		if (p)
			*p='\0';
		if (!*str || !strcasecmp(str,"---"))
			continue;
		p=str;
		while (isspace(*p))
			p++;
		if (!*p)
			continue;
		inhalt=strchr(p,'=');
		if (!inhalt)	
			continue;
		*inhalt++='\0';
		strtrim(inhalt);
		strtrim(p);
		if (!strcasecmp(p,"kuerzel")|| !strcasecmp(p,"kurz"))
		{
			if (new)
			{
				info_add_new(new);
				new=NULL;
			}
			if (*inhalt)
			{
				new=calloc(1,sizeof(minfo_t));
				if (!new)
				{
					log(LOG_ERR, _("out of memory"));
					fclose(f);
					return 1;
				}
				new->kuerzel=strdup(inhalt);
				if (!new->kuerzel)
				{
					log(LOG_ERR, _("out of memory"));
					fclose(f);
					return 1;
				}
			}
		}
		else if (!new)
			continue;
		else if (!strcasecmp(p,"text"))
		{
			new->kurztext=strdup(inhalt);
			if (!new->kurztext)
			{
				log(LOG_ERR, _("out of memory"));
				fclose(f);
				return 1;
			}
		}
		else if (!strcasecmp(p,"crc"))
			new->last_crc=strtol(inhalt,NULL,10);
		else if (!strcasecmp(p,"pause"))
			new->pause=strtoul(inhalt,NULL,10);
		else if (!strcasecmp(p,"zuletzt"))
			new->last_date=(time_t) strtoul(inhalt,NULL,10);
		else if (!strcasecmp(p,"mit-crc"))
			new->needs_crc=parse_bool(inhalt);
		else if (!strcasecmp(p,"diff"))
			new->send_diff=parse_bool(inhalt);
		else if (!strcasecmp(p,"mail"))
			new->send_mail=parse_bool(inhalt);
		else if (!strcasecmp(p,"cmd"))
			new->cmd=strdup(inhalt);
		else if (!strcasecmp(p,"bestellt")||!strcasecmp(p,"bestellen") )
			new->subscribed=parse_bool(inhalt);
	}
	fclose(f);
	if (new)
	{
		info_add_new(new);
	}
	return 0;
}

minfo_t *info_get_anker(void)
{
	return anker;
}

int info_write_list(void)
{
	char fname[PATH_MAX];
	minfo_t *akt;
	FILE *f;
	if (!anker)
		return 0;
	if (!dirty)
		return 0;
	sprintf(fname,"%s/%s/etc/info.list",opts.spoolbase,action);
	f=fopen(fname,"w");
	if (!f)
	{
		log(LOG_WARNING,_("cannot open %s for writing: %s"),
			fname,strerror(errno));
		return 0; /* naja, nur ein kleinerer Fehler! */
	}
	akt=anker;
	while(akt)
	{
		fprintf(f,"Kuerzel=%s\n",akt->kuerzel);
		if (akt->kurztext)
			fprintf(f,"\ttext=%s\n",akt->kurztext);
		fprintf(f,"\tcrc=%lu\n",akt->last_crc);
		fprintf(f,"\tpause=%ld\n",akt->pause);
		fprintf(f,"\tzuletzt=%ld\n",akt->last_date);
		fprintf(f,"\tmit-crc=%s\n",akt->needs_crc ? "true" : "false");
		fprintf(f,"\tmail=%s\n",akt->send_mail ? "true" : "false");
		if (akt->cmd)
			fprintf(f,"\tcmd=%s\n",akt->cmd);
		fprintf(f,"\tdiff=%s\n",akt->send_diff ? "true" : "false");
		fprintf(f,"\tbestellt=%s\n",akt->subscribed ? "true" : "false");
		akt=akt->next;
	}
	fclose(f);
	return 1;
}

void info_add_new(minfo_t *new)
{
	new->next=anker;
	anker=new;
	info_modified(new);
}

minfo_t *info_find(char *kuerzel)
{
	minfo_t *il;

	il=anker;
	while (il)
	{
		if (!strcasecmp(il->kuerzel,kuerzel))
			return il;
		il=il->next;
	}
	/* nicht gefunden */
	return NULL;
}

void info_new_crc(char *kuerzel, long crc)
{
	minfo_t *new,*il;

	il=info_find(kuerzel);
	if (il)
	{
		/* in jedem Fall speichern */
		dirty=TRUE;
		il->last_crc=crc;
		il->last_date=time(NULL);
		return;
	}
	/* nicht gefunden */

	/* 
	 * Also haben wir eine CRC fr ein unbekanntes und nicht bestelltes
	 * Infofile bekommen - merkwrdig, aber wahr. (denkbar ist, da
	 * jemand das Infile editiert hat)
	 */
	
	new=calloc(1,sizeof(minfo_t));
	if (!new)
		return;
	new->kuerzel=strdup(kuerzel);
	if (!new->kuerzel)
	{
		free(new);
		return;
	}
	new->last_crc=crc;    		/* crc auslesen */
	new->pause=0;			/* Annahme: Es soll bestellt werden */
	new->subscribed=1;
	new->last_date=time(NULL);	/* Datum des letzten Erhalts */
	info_add_new(new);
}

/*
 * dirty-Flag setzen, gewisse Defaults setzen
 */
void info_modified(minfo_t *mod)
{
	if (!mod->needs_crc)
	{
		mod->send_mail=1;
		mod->send_diff=0;
	}
	dirty=TRUE;
}


/*
 * returncodes:
 * 0: nothing changed
 * 1: new infofile or old data changed
 */
static int 
work_iti_infofile(char *kuerzel, char *kurztext, long needs_crc)
{
	minfo_t *found;
	found=info_find(kuerzel);
	if (!found)
	{	
		minfo_t *new;
		new=XCALLOC(1,sizeof(minfo_t));

		new->kuerzel=XSTRDUP(kuerzel);
		new->kurztext=XSTRDUP(kurztext);
		new->needs_crc=needs_crc;
		if (needs_crc)
			new->last_crc=-1;
		if (opts.autoinfosubscribe)
			new->subscribed=1;
		info_add_new(new);
		return 1;
	}
	else
	{
		/* haben sich eventuell Daten gendert? */
		int changed=0;
		if (found->needs_crc!=needs_crc)
		{
			found->needs_crc=needs_crc;
			if (needs_crc)
				found->last_crc=-1; /* beim naechsten Mal bestellen */
			changed=1;
		}
		if (found->kurztext && kurztext)
		{
			if (strcmp(found->kurztext,kurztext))
			{
				free(found->kurztext);
				found->kurztext=XSTRDUP(kurztext);
			}
		}
		else if (!found->kurztext && kurztext)
		{
			found->kurztext=XSTRDUP(kurztext);
		}
		if (changed)
		{
			info_modified(found);
			return 1;
		}
		return 0;
	}
}

static int
work_new_iti(char *iti)
{
	char *p;
	int changes=0;
	char kuerzel[256];
	char kurztext[1024];
	int needs_crc;

	char *thisline;
	char *nextline;

	*kuerzel='\0';
	*kurztext='\0';
	needs_crc=0;

	thisline=iti;
	while (*thisline)
	{
		char old=0;
		char *oldpos=NULL;
		p=strpbrk(thisline,"\r\n");
		if (p)
		{
			old=*p;
			oldpos=p;
			*p++='\0';
			while (*p=='\r' || *p=='\n')
				p++;
			nextline=p;
		}
		else
			nextline=thisline+strlen(thisline);
		switch(thisline[0])
		{
		case '#':	/* Name des Infofiles */
			if (*kuerzel)
			{
				if (work_iti_infofile(kuerzel, kurztext, needs_crc))
					changes++;
				*kuerzel='\0';
				*kurztext='\0';
				needs_crc=0;
			}
			strcpy(kuerzel,thisline+1);
			break;
		case ':':	/* Beschreibung des Infofiles */
			strcpy(kurztext,thisline+1);
			break;
		case 'F':
			if (maus_get_tauschflag(thisline+1,'C')=='+')
				needs_crc=1;
			break;
		}
		if (oldpos)
			*oldpos=old;
		thisline=nextline;
	}
	if (*kuerzel)
	{
		if (work_iti_infofile(kuerzel, kurztext, needs_crc))
			changes++;
	}
	return changes;
}


void
info_work_new_infofile(metamsg_t *meta, minfo_t *info)
{
	char fname[PATH_MAX];
	/* Wir haben ein Infofile bekommen. Der Inhalt steht in 
	 * meta->body, alle notwendigen Daten, es als Mail zu verschicken,
	 * stehen in meta.
	 */

	sprintf(fname,"%s/%s/infofiles/%s",opts.spoolbase,action,
	        info->kuerzel);

	if (!strcasecmp(info->kuerzel,"ITI"))
		work_new_iti(meta->body.start); /* Eventuelle neue ITI behandeln */

	/* Soll ein Diff gesendet werden? Das geht natrlich nur, wenn 
	 * schon ein solches Infofile da war.
	 */
	if (info->send_diff && !access(fname,R_OK)
		&& (opts.diffcmd && *opts.diffcmd))
	{
		/* diffs sind eine schoene Sache. Man bekommt sie auch sehr 
		 * einfach hin:
		 */
		FILE *wfd;
		char cmd[1024+PATH_MAX];
		sprintf(cmd,"%s %s - | mail -s 'MAUSINFO %s (diff)' '%s'",
			opts.diffcmd,fname,info->kuerzel,opts.mailuserdomain); 
		wfd=popen(cmd,"w");
		if (!wfd)
		{
			log(LOG_WARNING,_("cannot diff infofile %s (%s)"),
				info->kuerzel,strerror(errno));
		}
		else
		{
			/* Das interne Datenformat trennt die Zeilen mit \r\n, um
			 * nntp-Transfer zu beschleunigen und zu vereinfachen. Hier
			 * wird es dafr etwas langsamer.
			 * Tja, man kann nicht alles haben ...
			 */
			char *p;
			p=meta->body.start;
			while (*p)
			{
				if (*p!='\r')
					putc(*p,wfd);
				p++;
			}
			pclose(wfd);
			/* Und nun ist der diff unterwegs */
		}
	} /* Diff anfertigen? */
	if (info->cmd)
	{
		char cmd[1024+PATH_MAX];
		FILE *wfd;
		sprintf(cmd,"%s",info->cmd);
		wfd=popen(cmd,"w");
		if (!wfd)
		{
			log(LOG_WARNING,_("cannot send infofile %s via %s: %s"),
				info->kuerzel,info->cmd,strerror(errno));
		}
		else
		{
			/* Das interne Datenformat trennt die Zeilen mit \r\n, um
			 * nntp-Transfer zu beschleunigen und zu vereinfachen. Hier
			 * wird es dafr etwas langsamer.
			 * Tja, man kann nicht alles haben ...
			 */
			char *p;
			p=meta->body.start;
			while (*p)
			{
				if (*p!='\r')
					putc(*p,wfd);
				p++;
			}
			pclose(wfd);
		}
	}
	if (info->send_mail)
	{
		/* Das Infofile als Mail an den User senden. Das wird
		 * recht einfach erreicht, nmlich durch Eintrag einer 
		 * Empfngeradresse, wodurch die Nachricht spter als
		 * normale Mail versendet wird.
		 */
		/* falls das nicht klappt, ist es auch nicht so frchterlich
		 * schlimm.
		 */
		meta->env_to=strdup(opts.mailuserdomain);
	}
	/* Sichern? */
	if (info->send_diff || info->needs_crc || !info->send_mail)
	{
		FILE *f;
		f=fopen(fname,"w");
		if (!f)
		{
			log(LOG_ERR,_("cannot save infofile %s (%s)"),
				info->kuerzel,strerror(errno));
		}
		else
		{
			char *p;
			p=meta->body.start;
			while (p && *p)
			{
				if (*p!='\r')
					putc(*p,f);
				p++;
			}
			if (!meta->body.start)
				fprintf(f,_("Mauscon: infofile without text\n"));
			fclose(f);
		}
	}
}

