/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1994 1995 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "config.h"

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdlib.h>
#include <syslog.h>
#include <time.h>

#include "mausconlib.h"
#include "mauscon.h"
#include "metamsg.h"
#include "msg_io.h"


#define RES 10
static char *reserve[RES];

mio_t *
mio_alloc(mio_base_t *format)
{
	mio_t *neu;
	int i;

	for (i=0;i<RES;i++) if (!reserve[i]) reserve[i]=strdup("out of memory");

	if (!format)
		return NULL;

	neu=malloc(sizeof(mio_t));
	if (!neu)
		return NULL;

	neu->priv_data=NULL;
	neu->errtext=NULL;
	neu->fn=format;
	neu->fn->usecount++;
	return neu;
}

void
mio_free(mio_t *ch)
{
	int i;

	for (i=0;i<RES;i++) {
		if (!reserve[i]) 
			reserve[i]=strdup(_("out of memory"));
	}

	if (ch->fn)
		ch->fn->usecount--;
	free(ch->errtext);
	free(ch);
}

void
mio_set_errtext (mio_t *chan, const char *format,...)
{
	int ret;
	int i;
	va_list ap;
	if (chan->errtext)
	{
		free(chan->errtext);
		chan->errtext=NULL;
	}
	va_start (ap, format);
	ret=vasprintf(&chan->errtext,format,ap);
	va_end (ap);
	if (ret<1)
	{
		for (i=0;i<RES;i++) 
		{
			if (reserve[i]) 
			{
				chan->errtext=reserve[i];
				reserve[i]=NULL;
				return;
			}
		}
		log(LOG_CRIT,_("out of memory"));
		exit(1);
	}
	for (i=0;i<RES;i++) {
		if (!reserve[i]) 
			reserve[i]=strdup(_("out of memory"));
	}
}
