/* msg_io.h : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#ifndef MSG_IO_H
#define MSG_IO_H

/*
 * Import:
 *   MIO_OK:    eine Nachricht korrekt gelesen
 *   MIO_RETRY: diese eine Nachricht ist unretbar verloren (z.B. eine 
 *              Nachricht mit korruptem Header im Newsbatch)
 *   MIO_ERROR: die Nachricht konnte nicht gelesen werden, weiteres Lesen
 *              nicht mglich (zerstrter Newsbatch z.B.)
 *   MIO_EOF:   da war keine Nachricht mehr ...
 * Export:
 *   MIO_OK:    Nachricht korrekt geschrieben.
 *   MIO_RETRY: Probleme beim Schreiben dieser Nachricht, aber ein
 *              weiterer Versuch ist mglich, sollte allerdings 
 *              verzgert erfolgen.
 *   MIO_ERROR: die Nachricht ist unzustellbar und mu gebounced werden.
 * Open:
 *   MIO_OK:    korrekt geffnet.
 *   MIO_RETRY: temporrer Fehler.
 *   MIO_ERROR: Was auch immer es ist, es lt sich nicht ffnen.
 */
typedef enum {
	MIO_OK,
	MIO_RETRY,
	MIO_ERROR,
	MIO_EOF
} mio_retcode_t;

/*
 * Private Daten einer Transport/Konvertert-instanz
 */
typedef void mio_private_t;

/*
 * Ein Kanal ...
 */
typedef struct mio_t
{
	/* Letzte Fehlermeldung */
	char *errtext;
	int errcode;
	struct mio_base_t *fn;
	/* Private Daten des Kanals */
	void *priv_data;
} mio_t;


/*
 * Diese Struktur beschreibt die fr ein Nachrichtenformat mglichen
 * Funktionen.
 */
typedef struct mio_base_t
{
	/* Name - "MausTausch" oder "rmail" ... */
	const char *name;

	/* 
	 * Falls der Speicher nicht ausreicht, wird NULL zurckgeliefert.
	 * Falls ein anderer Fehler auftrat, wird eine mio_t zurckgeliefert,
	 * in dem "errtext" gesetzt ist. Der Kanal ist in dem Fall zu nichts 
	 * weiter * nutzbar als die Fehlermeldung auszulesen und danach die
	 * "close"-Funktion aufzurufen.
	 */
	mio_retcode_t (*open)   (const char *fname, const char *mode,
							mio_t **neu, char *errbuf);
	mio_retcode_t (*import) (mio_t *, metamsg_t *);
	mio_retcode_t (*export) (mio_t *, metamsg_t *);
	mio_retcode_t (*close)  (mio_t *);

	int usecount;

} mio_base_t;

void mio_set_errtext(mio_t *,const char *format,...);

/* nur zum internen Gebrauch in den Modulen bestimmt */
mio_t * mio_alloc(mio_base_t *format);
void mio_free(mio_t *);

	/* MausTausch (I/O) */
extern mio_base_t mio_maustausch;
	/* Internetmail, direktes SMTP (O) */
extern mio_base_t mio_smtp;
	/* Internetmail, SMTP-Mailbatchfiles (O) */
extern mio_base_t mio_batch_smtp;
	/* Usenetnews, NNTP (zur Zeit nur eine Richtung) (O) */
extern mio_base_t mio_nntp;
	/* Usenetnews, rnews (zur Zeit nur eine Richtung) (O) */
extern mio_base_t mio_rnews;
	/* Usenetnews, hot-rnews (rnews wird aufgerufen, 
	 * wenn der Batch geschrieben wurde) (zur Zeit nur eine Richtung) (O) */
extern mio_base_t mio_hot_rnews;

	/* Internetmail, rmail (I) */
extern mio_base_t mio_rmail;
	/* Usenetnews, nntp (O) */
extern mio_base_t mio_nntp;
	/* Das von suck verwendete Format fr Usenetnews (I/O) */
extern mio_base_t mio_suck;

#endif
