/* MausConnect: Nachrichtenformatskonverter fuer das MausNet
   Copyright (C) 1994 1995 Michael Kraehe, Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   route.c: Routing fr Mauscon
*/

#include "config.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include <time.h>
#include <ctype.h>
#include <limits.h>
#include <fnmatch.h>

#include "mausconlib.h"
#include "mauscon.h"
#include "route.h"

typedef struct route_struct
{
	char *action;
	char *from;
	char *dest;
	struct route_struct *next;
} route_t;

route_t *anker;

int route_show(const char *which)
{
	route_t *a;
	if (!anker)
		return 1;
	a=anker;
	while (a)
	{
		if (!which || !strcasecmp(which,a->action) 
			|| !strcasecmp(which,"route"))
		{
			printf("%s: %s / %s\n",	
				   a->action, a->from, a->dest);
		}
		a=a->next;
	}
	return 0;
}

int route_open(const char *libdir)
{	
	FILE *f;
	route_t *last=NULL;
	route_t *new;
	char buf[10240];
	char *pa,*pf,*pd,*p;
	char fname[PATH_MAX];
	sprintf(fname,"%s/routes",libdir);
	f=fopen(fname,"r");
	if (!f)
		return 1;
	while (fgets(buf,sizeof(buf),f))
	{
		pa=buf;
		while(*pa && isspace(*pa))
			pa++;
		if (!*pa || *pa=='#')
			continue;
		pf=strpbrk(pa,"\t ");
		if (!pf)
			continue;
		*pf++='\0';
		pd=strpbrk(pf,"\t ");
		if (!pd)
			continue;
		*pd++='\0';
		p=pd+strlen(pd)-1;
		while (isspace(*p) && p>=pd)
			p--;
		if (p<pd)
			continue;
		p[1]='\0';
		new=XMALLOC(sizeof(route_t));
		new->action=XSTRDUP(pa);
		new->from=XSTRDUP(pf);
		new->dest=XSTRDUP(pd);
		if (!last)
			anker=new;
		else
			last->next=new;
		last=new;
	}
	fclose(f);
	return 0;
}

void route_close(void)
{
	route_t *akt,*next;
	akt=anker;
	while (akt)
	{
		free(akt->dest);
		free(akt->from);
		free(akt->action);
		next=akt->next;
		free(akt);
		akt=next;
	}
	anker=NULL;
}

static int
inlist (const char *what, char *list)
{
	char *p;
	int ret;
	do
	{
		p=strchr(list,':');
		if (!p)
			return !fnmatch(list,what,FNM_CASEFOLD);
		*p='\0';
		ret=!fnmatch(list,what,FNM_CASEFOLD);
		*p=':';
		if (ret)
			return ret;
		list=p+1;
	} while (*list);
	return 0;
}

char *
route_it(const char *from, const char *dest)
{
	route_t *akt;
	char *p;

	p=strchr(dest,'@');
	if (p)
		dest=p+1;
	akt=anker;

	while (akt)
	{
		if (inlist(from,akt->from))
		{
			if (!dest)
				/* Oeffentliche Nachricht */
				return akt->action;
			else if (inlist(dest,akt->dest))
				return akt->action;
		}
		akt=akt->next;
	}
	return NULL;
}

